USE [AdventureWorks2012]
GO

IF NOT EXISTS (SELECT * FROM sys.tables where name like 'PartitionSwap')
	CREATE TABLE dbo.PartitionSwap (PartitionTable varchar(200), RangeType varchar(10), PartitionScheme varchar(300), PartitionFunction varchar(300), PartitionNumber int, PartitionRows bigint, FileGroupName varchar(200), BoundaryValue sql_variant, CollectDate datetime default(getdate()), EndDate datetime, Processed bit default(0))

/* Step 1 - Insert into meta data */
BEGIN
		DECLARE @PartitionSchemeName varchar(200), @PartitionNumber int;
		SELECT @PartitionSchemeName = 'Demo3_Scheme', @PartitionNumber = 2;

		WITH PSCTE AS (SELECT 
			  SPS.name AS PartitionSchemeName
			, CASE WHEN SDD.destination_id <= SPF.fanout THEN SDD.destination_id
				ELSE NULL END AS PartitionID
			, SPF.name AS PartitionFunctionName
			,SPF.function_id
			, SPRV.value AS BoundaryValue
			, CASE WHEN SDD.destination_id > SPF.fanout THEN 1 
				ELSE 0 END AS NextUsed
			, SF.name AS FileGroup
		FROM sys.partition_schemes AS SPS
		JOIN sys.partition_functions AS SPF 
			ON SPS.function_id = SPF.function_id
		JOIN sys.destination_data_spaces AS SDD 
			ON SDD.partition_scheme_id = SPS.data_space_id
		JOIN sys.filegroups AS SF 
			ON SF.data_space_id = SDD.data_space_id
		LEFT JOIN sys.partition_range_values AS SPRV 
			ON SPRV.function_id = SPF.function_id
			AND SDD.destination_id = 
		CASE WHEN SPF.boundary_value_on_right = 0 THEN SPRV.boundary_id
			ELSE SPRV.boundary_id + 1 END 
		WHERE SPS.name like 'Demo3_Scheme')
		INSERT INTO dbo.PartitionSwap (PartitionTable, RangeType, PartitionScheme, PartitionFunction, PartitionNumber, PartitionRows, FileGroupName, BoundaryValue)
		SELECT 
			OBJECT_NAME(SI.object_id) AS PartitionedTable
			,CASE WHEN PF.boundary_value_on_right = 0 THEN 'LEFT' ELSE 'RIGHT' END AS RangeType
			, DS.name AS PartitionSchemeName
			, PF.name AS PartitionFunction
			, P.partition_number AS PartitionNumber
			, P.rows AS PartitionRows
			, FG.name AS FileGroupName
			, SPRV.BoundaryValue AS BoundaryValue
	
		FROM sys.partitions AS P
		JOIN sys.indexes AS SI
			ON P.object_id = SI.object_id AND P.index_id = SI.index_id 
		JOIN sys.data_spaces AS DS
			ON DS.data_space_id = SI.data_space_id
		JOIN sys.partition_schemes AS PS
			ON PS.data_space_id = SI.data_space_id
		JOIN sys.partition_functions AS PF
			ON PF.function_id = PS.function_id 
		JOIN sys.destination_data_spaces AS DDS
			ON DDS.partition_scheme_id = SI.data_space_id 
			AND DDS.destination_id = P.partition_number
		JOIN sys.filegroups AS FG
			ON DDS.data_space_id = FG.data_space_id
		JOIN PSCTE AS SPRV 
			ON (SPRV.PartitionFunctionName = PF.name and SPRV.PartitionID = P.partition_number)
		WHERE DS.type = 'PS'
		AND OBJECTPROPERTYEX(SI.object_id, 'BaseType') = 'U'
		AND SI.type IN(0,1)
		AND ds.name like @PartitionSchemeName
		AND P.partition_number = @PartitionNumber;
END

SELECT * FROM dbo.PartitionSwap
/****** Step two: Generate Staging Table for meta data swap ********

Run with powershell
C:\demo\TablePartitioning\ManagePartition\ManagePartition.exe /C:CreateStagingFull /d:AdventureWorks2012 /s:Sales /t:SalesOrderHeaderEnlarged_PT /A:SalesOrderHeaderEnlarged_Staging /p:2 /f:"C:\demo\TablePartitioning\ManagePartition\scrpt.sql" /O:o

*/

/* 
	Step 3 Meta data swap 
	** No data moved, just requires schema lock **
*/
exec dbo.PartitionQuery

ALTER TABLE [Sales].[SalesOrderHeaderEnlarged_PT] 
	SWITCH PARTITION 2 TO [Sales].[SalesOrderHeaderEnlarged_Staging];

SELECT COUNT(*) from [Sales].[SalesOrderHeaderEnlarged_Staging];
exec dbo.PartitionQuery

/*** Step 4 - Merge Partition 1 and Partition 2. 
	They are both empty so no data movement occurs  ***/
BEGIN
	DECLARE @MergeVal datetime
	SELECT TOP 1 @MergeVal = CAST(BoundaryValue as datetime) 
	FROM dbo.PartitionSwap ps
	WHERE ps.Processed = 0
	AND ps.PartitionTable like 'SalesOrderHeaderEnlarged_PT'
	AND ps.PartitionNumber = 2
	ORDER BY ps.CollectDate desc
	
	SELECT @MergeVal

	ALTER PARTITION FUNCTION Demo3_Function()
	MERGE RANGE (@MergeVal)
END

/*** Step 5 - Mark empty partition from merge as next used partition ***/
BEGIN
	DECLARE @FileGroup varchar(200), @SQLStmt nvarchar(max)
	SELECT TOP 1 @FileGroup = FileGroupName 
	FROM dbo.PartitionSwap ps
	WHERE ps.Processed = 0
	AND ps.PartitionTable like 'SalesOrderHeaderEnlarged_PT'
	AND ps.PartitionNumber = 2
	ORDER BY ps.CollectDate desc

	SELECT @FileGroup
	
	SET @SQLStmt = 'ALTER PARTITION SCHEME Demo3_Scheme NEXT USED '+ @FileGroup 
	EXEC sp_executeSQL @SQLStmt
END

/** Step 6 - Create new Partition **/
BEGIN
	DECLARE @SplitValue datetime

	SELECT 
		@SplitValue = MAX(CAST(PRV.value AS datetime))
	FROM sys.partition_functions AS PF
	JOIN sys.partition_parameters AS PP 
		ON PF.function_id = PP.function_id
	JOIN sys.types AS T1 
		ON T1.system_type_id = PP.system_type_id
	JOIN sys.types AS T2 
		ON T2.user_type_id= PP.user_type_id
	JOIN sys.partition_range_values AS PRV 
		ON PP.function_id = PRV.function_id 
	   AND PP.parameter_id = PRV.parameter_id
	WHERE PF.name like 'Demo3_Function';

	--- ADDING 6 MONTHS TO LAST BOUNDARY VALUE ---
	SELECT @SplitValue = DATEADD(month, 6, @SplitValue)
	IF @SplitValue IS NOT NULL
	BEGIN
		ALTER PARTITION FUNCTION Demo3_Function()
		SPLIT RANGE (@SplitValue);
	END -- IF Statement
END
/** Step 7 - Update Processed flag **/	
	UPDATE ps
	SET Processed = 1, EndDate = getdate()
	FROM dbo.PartitionSwap ps
	WHERE ps.Processed = 0
	AND ps.PartitionTable like 'SalesOrderHeaderEnlarged_PT'
	AND ps.PartitionNumber = 2
	AND ps.CollectDate = (SELECT MAX(CollectDate) FROM dbo.PartitionSwap ps2
						  WHERE ps2.Processed = ps.Processed
						  AND ps2.PartitionTable = ps.PartitionTable
						  AND ps2.PartitionNumber = ps.PartitionNumber)

/** Step 8 - Archive staging table or truncate it **/
DROP TABLE [Sales].[SalesOrderHeaderEnlarged_Staging]